#!/usr/bin/env python3

import argparse
import collections
import re
import subprocess
import sys

parser = argparse.ArgumentParser()
parser.add_argument('full_scopefile',
                    type=argparse.FileType('r'),
                    metavar='<full scopefile>',
                   )
parser.add_argument('alloc_hostnames',
                    type=argparse.FileType('r'),
                    metavar='<allocated hostnames>',
                    nargs='?',
                    help='File with a list of hostnames of nodes in an allocation. If not present, the output of scontrol show hostnames will be used instead.',
                   )
parser.add_argument('-o', '--output',
                    type=argparse.FileType('w'),
                    required=False,
                    help='Output scopefile. If not given, use stdout.',
                    default=sys.stdout,
                   )
parser.add_argument('-s', '--node-suffix',
                    required=False,
                    help='Suffix to add to nodename in output scopefile.',
                    default='',
                   )

args = parser.parse_args()

allocated_nodes = {}
if not args.alloc_hostnames:
    cp = subprocess.run(["scontrol", "show", "hostnames"],
                        check=True,
                        stdout=subprocess.PIPE,
                        universal_newlines=True,
                       )
    lines = cp.stdout.splitlines()
else:
    lines = args.alloc_hostnames.readlines()

if len(lines) == 0:
    raise RuntimeError('Empty list of allocated nodes')

for l in lines:
    l = l.strip()
    n = re.split('[.\s]', l, 1)[0].lower()
    # l - nodename from allocated_nodes file
    # n - l stripped up to first dot and lowercased
    # we use n for comparing nodes and l for output files
    allocated_nodes[n] = l + args.node_suffix

scopes = collections.defaultdict(list)
# max_scope_size is max size of a scope after cutting scopes to allocated nodes
# It can't be more than a total number of nodes, so use it as a start value
max_scope_size = 0
for l in args.full_scopefile:
    max_scope_size += 1
    l = l.strip()
    node, scope, scope_num = l.split(',')
    node = re.split('[.\s]', node, 1)[0].lower()
    if node in allocated_nodes:
        scopes[scope_num].append((allocated_nodes[node], scope))

for scope_num, nodes in scopes.items():
    max_scope_size = min(max_scope_size, len(nodes))

new_scope_num = 0
for scope_num, nodes in scopes.items():
    new_scope_num += 1
    for nodename, scope in scopes[scope_num][0:max_scope_size]:
        print('{},{},{}'.format(nodename,scope,new_scope_num), file=args.output)
